/*
 	File:		GXErrors.h
 
 	Contains:	QuickDraw GX error constants and debugging routines
 
 	Version:	Technology:	Quickdraw GX 1.1
 				Package:	Universal Interfaces 2.1 in MPW Latest on ETO #18
 
 	Copyright:	 1984-1995 by Apple Computer, Inc.
 				All rights reserved.
 
 	Bugs?:		If you find a problem with this file, use the Apple Bug Reporter
 				stack.  Include the file and version information (from above)
 				in the problem description and send to:
 					Internet:	apple.bugs@applelink.apple.com
 					AppleLink:	APPLE.BUGS
 
*/

#ifndef __GXERRORS__
#define __GXERRORS__


#ifndef __CONDITIONALMACROS__
#include <ConditionalMacros.h>
#endif

#ifndef __MIXEDMODE__
#include <MixedMode.h>
#endif
/*	#include <Types.h>											*/

#ifndef __GXTYPES__
#include <GXTypes.h>
#endif
/*	#include <GXMath.h>											*/
/*		#include <FixMath.h>									*/

#ifdef __cplusplus
extern "C" {
#endif

#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=mac68k
#endif

#if PRAGMA_IMPORT_SUPPORTED
#pragma import on
#endif

#if defined(__MWERKS__) && GENERATING68K
	#pragma push
	#pragma pointers_in_D0
#endif
 
#define graphicsErrorsIncludes
/* old header = graphics errors */
 

enum {
	gxFirstSystemError			= -27999,						/* 0xffff92a1 */
	gxFirstFatalError			= -27999,
	gxLastFatalError			= -27951,
	gxFirstNonfatalError		= -27950,
	gxFirstFontScalerError		= -27900,
	gxLastFontScalerError		= -27851,
	gxFirstParameterError		= -27850,
	gxFirstImplementationLimitError = -27800,
	gxFirstSystemDebuggingError	= -27700,
	gxLastSystemError			= -27000,						/* 0xffff9688 */
	gxFirstLibraryError			= 1048576,						/* 0x00100000 */
	gxLastLibraryError			= 2097151,						/* 0x001fffff */
	gxFirstAppError				= 2097152,						/* 0x00200000 */
	gxLastAppError				= 4194303,						/* 0x003fffff */
	gxFirstSystemWarning		= -26999,						/* 0xffff9689 */
	gxFirstResultOutOfRangeWarning = -26950,
	gxFirstParameterOutOfRangeWarning = -26900,
	gxFirstFontScalerWarning	= -26850,
	gxFirstSystemDebuggingWarning = -26700,
	gxLastSystemWarning			= -26000,						/* 0xffff9a70 */
	gxFirstLibraryWarning		= 4194304,						/* 0x00400000 */
	gxLastLibraryWarning		= 5242879,						/* 0x004fffff */
	gxFirstAppWarning			= 5242880,						/* 0x00500000 */
	gxLastAppWarning			= 7340031,						/* 0x006fffff */
	gxFirstSystemNotice			= -25999,						/* 0xffff9a71 */
	gxLastSystemNotice			= -25500,						/* 0xffff9c64 */
	gxFirstLibraryNotice		= 7340032,						/* 0x00700000 */
	gxLastLibraryNotice			= 7602175,						/* 0x0073ffff */
	gxFirstAppNotice			= 7602176,						/* 0x00740000 */
	gxLastAppNotice				= 8388607						/* 0x007fffff */
};

enum gxGraphicErrors {
/* truly fatal errors */
	out_of_memory				= gxFirstFatalError,
	internal_fatal_error		= gxFirstFatalError + 1,
	no_outline_font_found		= gxFirstFatalError + 2,
	not_enough_memory_for_graphics_client_heap = gxFirstFatalError + 3,
	could_not_create_backing_store = gxFirstFatalError + 4,
/* internal errors */
	internal_error				= gxFirstNonfatalError,
	internal_font_error			= gxFirstNonfatalError + 1,
	internal_layout_error		= gxFirstNonfatalError + 2,
/* recoverable errors */
	could_not_dispose_backing_store = internal_layout_error + 2,
	unflattening_interrupted_by_client = internal_layout_error + 3,
/* font manager errors */
	font_cannot_be_changed		= internal_layout_error + 4,
	illegal_font_parameter		= internal_layout_error + 5,
/* gxFont scaler errors */
	null_font_scaler_context	= gxFirstFontScalerError,
	null_font_scaler_input		= gxFirstFontScalerError + 1,
	invalid_font_scaler_context	= gxFirstFontScalerError + 2,
	invalid_font_scaler_input	= gxFirstFontScalerError + 3,
	invalid_font_scaler_font_data = gxFirstFontScalerError + 4,
	font_scaler_newblock_failed	= gxFirstFontScalerError + 5,
	font_scaler_getfonttable_failed = gxFirstFontScalerError + 6,
	font_scaler_bitmap_allocation_failed = gxFirstFontScalerError + 7,
	font_scaler_outline_allocation_failed = gxFirstFontScalerError + 8,
	required_font_scaler_table_missing = gxFirstFontScalerError + 9,
	unsupported_font_scaler_outline_format = gxFirstFontScalerError + 10,
	unsupported_font_scaler_stream_format = gxFirstFontScalerError + 11,
	unsupported_font_scaler_font_format = gxFirstFontScalerError + 12,
	font_scaler_hinting_error	= gxFirstFontScalerError + 13,
	font_scaler_rasterizer_error = gxFirstFontScalerError + 14,
	font_scaler_internal_error	= gxFirstFontScalerError + 15,
	font_scaler_invalid_matrix	= gxFirstFontScalerError + 16,
	font_scaler_fixed_overflow	= gxFirstFontScalerError + 17,
	font_scaler_api_version_mismatch = gxFirstFontScalerError + 18,
	font_scaler_streaming_aborted = gxFirstFontScalerError + 19,
	unknown_font_scaler_error	= gxFirstFontScalerError + 20,
/* bad parameters */
	parameter_is_nil			= gxFirstParameterError,
	shape_is_nil				= gxFirstParameterError + 1,
	style_is_nil				= gxFirstParameterError + 2,
	transform_is_nil			= gxFirstParameterError + 3,
	ink_is_nil					= gxFirstParameterError + 4,
	transferMode_is_nil			= gxFirstParameterError + 5,
	color_is_nil				= gxFirstParameterError + 6,
	colorProfile_is_nil			= gxFirstParameterError + 7,
	colorSet_is_nil				= gxFirstParameterError + 8,
	spoolProcedure_is_nil		= gxFirstParameterError + 9,
	tag_is_nil					= gxFirstParameterError + 10,
	type_is_nil					= gxFirstParameterError + 11,
	mapping_is_nil				= gxFirstParameterError + 12,
	invalid_viewDevice_reference = gxFirstParameterError + 13,
	invalid_viewGroup_reference	= gxFirstParameterError + 14,
	invalid_viewPort_reference	= gxFirstParameterError + 15,
/* implementation limits, these should be right before the debugging errors */
	number_of_contours_exceeds_implementation_limit = gxFirstImplementationLimitError,
	number_of_points_exceeds_implementation_limit = gxFirstImplementationLimitError + 1,
	size_of_polygon_exceeds_implementation_limit = gxFirstImplementationLimitError + 2,
	size_of_path_exceeds_implementation_limit = gxFirstImplementationLimitError + 3,
	size_of_text_exceeds_implementation_limit = gxFirstImplementationLimitError + 4,
	size_of_bitmap_exceeds_implementation_limit = gxFirstImplementationLimitError + 5,
	number_of_colors_exceeds_implementation_limit = gxFirstImplementationLimitError + 6,
	procedure_not_reentrant		= gxFirstImplementationLimitError + 7
};

enum {
/* internal debugging errors */
	functionality_unimplemented	= gxFirstSystemDebuggingError,
	clip_to_frame_shape_unimplemented = gxFirstSystemDebuggingError + 1,
/* font parameter debugging errors */
	illegal_font_storage_type	= gxFirstSystemDebuggingError + 2,
	illegal_font_storage_reference = gxFirstSystemDebuggingError + 3,
	illegal_font_attributes		= gxFirstSystemDebuggingError + 4,
/* parameter debugging errors */
	parameter_out_of_range		= gxFirstSystemDebuggingError + 5,
	inconsistent_parameters		= gxFirstSystemDebuggingError + 6,
	index_is_less_than_zero		= gxFirstSystemDebuggingError + 7,
	index_is_less_than_one		= gxFirstSystemDebuggingError + 8,
	count_is_less_than_zero		= gxFirstSystemDebuggingError + 9,
	count_is_less_than_one		= gxFirstSystemDebuggingError + 10,
	contour_is_less_than_zero	= gxFirstSystemDebuggingError + 11,
	length_is_less_than_zero	= gxFirstSystemDebuggingError + 12,
	invalid_client_reference	= gxFirstSystemDebuggingError + 13,
	invalid_graphics_heap_start_pointer = gxFirstSystemDebuggingError + 14,
	invalid_nongraphic_globals_pointer = gxFirstSystemDebuggingError + 15,
	colorSpace_out_of_range		= gxFirstSystemDebuggingError + 16,
	pattern_lattice_out_of_range = gxFirstSystemDebuggingError + 17,
	frequency_parameter_out_of_range = gxFirstSystemDebuggingError + 18,
	tinting_parameter_out_of_range = gxFirstSystemDebuggingError + 19,
	method_parameter_out_of_range = gxFirstSystemDebuggingError + 20,
	space_may_not_be_indexed	= gxFirstSystemDebuggingError + 21,
	glyph_index_too_small		= gxFirstSystemDebuggingError + 22,
	no_glyphs_added_to_font		= gxFirstSystemDebuggingError + 23,
	glyph_not_added_to_font		= gxFirstSystemDebuggingError + 24,
	point_does_not_intersect_bitmap = gxFirstSystemDebuggingError + 25,
	required_font_table_not_present = gxFirstSystemDebuggingError + 25,
	unknown_font_table_format	= gxFirstSystemDebuggingError + 26, /* the styles encoding is not present in the font */
	shapeFill_not_allowed		= gxFirstSystemDebuggingError + 27,
	inverseFill_face_must_set_clipLayer_flag = gxFirstSystemDebuggingError + 28,
	invalid_transferMode_colorSpace = gxFirstSystemDebuggingError + 29,
	colorProfile_must_be_nil	= gxFirstSystemDebuggingError + 30,
	bitmap_pixel_size_must_be_1	= gxFirstSystemDebuggingError + 31,
	empty_shape_not_allowed		= gxFirstSystemDebuggingError + 32,
	ignorePlatformShape_not_allowed = gxFirstSystemDebuggingError + 33,
	nil_style_in_glyph_not_allowed = gxFirstSystemDebuggingError + 34,
	complex_glyph_style_not_allowed = gxFirstSystemDebuggingError + 35,
	invalid_mapping				= gxFirstSystemDebuggingError + 36,
	cannot_set_item_shapes_to_nil = gxFirstSystemDebuggingError + 37,
	cannot_use_original_item_shapes_when_growing_picture = gxFirstSystemDebuggingError + 38,
	cannot_add_unspecified_new_glyphs = gxFirstSystemDebuggingError + 39,
	cannot_dispose_locked_tag	= gxFirstSystemDebuggingError + 40,
	cannot_dispose_locked_shape	= gxFirstSystemDebuggingError + 41,
/* restricted access */
	shape_access_not_allowed	= gxFirstSystemDebuggingError + 42,
	colorSet_access_restricted	= gxFirstSystemDebuggingError + 43,
	colorProfile_access_restricted = gxFirstSystemDebuggingError + 44,
	tag_access_restricted		= gxFirstSystemDebuggingError + 45,
	viewDevice_access_restricted = gxFirstSystemDebuggingError + 46,
	graphic_type_does_not_have_a_structure = gxFirstSystemDebuggingError + 47,
	style_run_array_does_not_match_number_of_characters = gxFirstSystemDebuggingError + 48,
	rectangles_cannot_be_inserted_into = gxFirstSystemDebuggingError + 49,
	unknown_graphics_heap		= gxFirstSystemDebuggingError + 50,
	graphics_routine_selector_is_obsolete = gxFirstSystemDebuggingError + 51,
	cannot_set_graphics_client_memory_without_setting_size = gxFirstSystemDebuggingError + 52,
	graphics_client_memory_too_small = gxFirstSystemDebuggingError + 53,
	graphics_client_memory_is_already_allocated = gxFirstSystemDebuggingError + 54,
	viewPort_is_a_window		= gxFirstSystemDebuggingError + 55,
/* wrong type/bad reference */
	illegal_type_for_shape		= gxFirstSystemDebuggingError + 56,
	shape_does_not_contain_a_bitmap = gxFirstSystemDebuggingError + 57,
	shape_does_not_contain_text	= gxFirstSystemDebuggingError + 58,
	picture_expected			= gxFirstSystemDebuggingError + 59,
	bitmap_is_not_resizable		= gxFirstSystemDebuggingError + 60,
	shape_may_not_be_a_bitmap	= gxFirstSystemDebuggingError + 61,
	shape_may_not_be_a_picture	= gxFirstSystemDebuggingError + 62,
	graphic_type_does_not_contain_points = gxFirstSystemDebuggingError + 63,
	graphic_type_does_not_have_multiple_contours = gxFirstSystemDebuggingError + 64,
	graphic_type_cannot_be_mapped = gxFirstSystemDebuggingError + 65,
	graphic_type_cannot_be_moved = gxFirstSystemDebuggingError + 66,
	graphic_type_cannot_be_scaled = gxFirstSystemDebuggingError + 67,
	graphic_type_cannot_be_rotated = gxFirstSystemDebuggingError + 68,
	graphic_type_cannot_be_skewed = gxFirstSystemDebuggingError + 69,
	graphic_type_cannot_be_reset = gxFirstSystemDebuggingError + 70,
	graphic_type_cannot_be_dashed = gxFirstSystemDebuggingError + 71,
	graphic_type_cannot_be_reduced = gxFirstSystemDebuggingError + 72,
	graphic_type_cannot_be_inset = gxFirstSystemDebuggingError + 73,
	shape_cannot_be_inverted	= gxFirstSystemDebuggingError + 74,
	shape_does_not_have_area	= gxFirstSystemDebuggingError + 75,
	shape_does_not_have_length	= gxFirstSystemDebuggingError + 76,
	first_glyph_advance_must_be_absolute = gxFirstSystemDebuggingError + 77,
	picture_cannot_contain_itself = gxFirstSystemDebuggingError + 78,
	viewPort_cannot_contain_itself = gxFirstSystemDebuggingError + 79,
	cannot_set_unique_items_attribute_when_picture_contains_items = gxFirstSystemDebuggingError + 80,
	layer_style_cannot_contain_a_face = gxFirstSystemDebuggingError + 81,
	layer_glyph_shape_cannot_contain_nil_styles = gxFirstSystemDebuggingError + 82,
/* validation errors */
	object_wrong_type			= gxFirstSystemDebuggingError + 83,
	shape_wrong_type			= gxFirstSystemDebuggingError + 84,
	style_wrong_type			= gxFirstSystemDebuggingError + 85,
	ink_wrong_type				= gxFirstSystemDebuggingError + 86,
	transform_wrong_type		= gxFirstSystemDebuggingError + 87,
	device_wrong_type			= gxFirstSystemDebuggingError + 88,
	port_wrong_type				= gxFirstSystemDebuggingError + 89,
/* validation cache errors */
	shape_cache_wrong_type		= gxFirstSystemDebuggingError + 90,
	style_cache_wrong_type		= gxFirstSystemDebuggingError + 91,
	ink_cache_wrong_type		= gxFirstSystemDebuggingError + 92,
	transform_cache_wrong_type	= gxFirstSystemDebuggingError + 93,
	port_cache_wrong_type		= gxFirstSystemDebuggingError + 94,
	shape_cache_parent_mismatch	= gxFirstSystemDebuggingError + 95,
	style_cache_parent_mismatch	= gxFirstSystemDebuggingError + 96,
	ink_cache_parent_mismatch	= gxFirstSystemDebuggingError + 97,
	transform_cache_parent_mismatch = gxFirstSystemDebuggingError + 98,
	port_cache_parent_mismatch	= gxFirstSystemDebuggingError + 99,
	invalid_shape_cache_port	= gxFirstSystemDebuggingError + 100,
	invalid_shape_cache_device	= gxFirstSystemDebuggingError + 101,
	invalid_ink_cache_port		= gxFirstSystemDebuggingError + 102,
	invalid_ink_cache_device	= gxFirstSystemDebuggingError + 103,
	invalid_style_cache_port	= gxFirstSystemDebuggingError + 104,
	invalid_style_cache_device	= gxFirstSystemDebuggingError + 105,
	invalid_transform_cache_port = gxFirstSystemDebuggingError + 106,
	invalid_transform_cache_device = gxFirstSystemDebuggingError + 107,
	recursive_caches			= gxFirstSystemDebuggingError + 108,
/* validation shape cache errors */
	invalid_fillShape_ownerCount = gxFirstSystemDebuggingError + 109,
	recursive_fillShapes		= gxFirstSystemDebuggingError + 110,
/* validation memory block errors */
	indirect_memory_block_too_small = gxFirstSystemDebuggingError + 111,
	indirect_memory_block_too_large = gxFirstSystemDebuggingError + 112,
	unexpected_nil_pointer		= gxFirstSystemDebuggingError + 113,
	bad_address					= gxFirstSystemDebuggingError + 114,
/* validation object errors */
	no_owners					= gxFirstSystemDebuggingError + 115,
	invalid_pointer				= gxFirstSystemDebuggingError + 116,
	invalid_seed				= gxFirstSystemDebuggingError + 117,
	invalid_frame_seed			= gxFirstSystemDebuggingError + 118,
	invalid_text_seed			= gxFirstSystemDebuggingError + 119,
	invalid_draw_seed			= gxFirstSystemDebuggingError + 120,
	bad_private_flags			= gxFirstSystemDebuggingError + 121,
/* validation path and polygon errors */
	invalid_vector_count		= gxFirstSystemDebuggingError + 122,
	invalid_contour_count		= gxFirstSystemDebuggingError + 123,
/* validation bitmap errors */
	bitmap_ptr_too_small		= gxFirstSystemDebuggingError + 124,
	bitmap_ptr_not_aligned		= gxFirstSystemDebuggingError + 125,
	bitmap_rowBytes_negative	= gxFirstSystemDebuggingError + 126,
	bitmap_width_negative		= gxFirstSystemDebuggingError + 127,
	bitmap_height_negative		= gxFirstSystemDebuggingError + 128,
	invalid_pixelSize			= gxFirstSystemDebuggingError + 129,
	bitmap_rowBytes_too_small	= gxFirstSystemDebuggingError + 130,
	bitmap_rowBytes_not_aligned	= gxFirstSystemDebuggingError + 131,
	bitmap_rowBytes_must_be_specified_for_user_image_buffer = gxFirstSystemDebuggingError + 132,
/* validation bitmap image errors */
	invalid_bitImage_fileOffset	= gxFirstSystemDebuggingError + 133,
	invalid_bitImage_owners		= gxFirstSystemDebuggingError + 134,
	invalid_bitImage_rowBytes	= gxFirstSystemDebuggingError + 135,
	invalid_bitImage_internal_flag = gxFirstSystemDebuggingError + 136,
/* validation text errors */
	text_bounds_cache_wrong_size = gxFirstSystemDebuggingError + 137,
	text_metrics_cache_wrong_size = gxFirstSystemDebuggingError + 138,
	text_index_cache_wrong_size	= gxFirstSystemDebuggingError + 139,
/* validation glyph errors */
	glyph_run_count_negative	= gxFirstSystemDebuggingError + 140,
	glyph_run_count_zero		= gxFirstSystemDebuggingError + 141,
	glyph_run_counts_do_not_sum_to_character_count = gxFirstSystemDebuggingError + 142,
	glyph_first_advance_bit_set_not_allowed = gxFirstSystemDebuggingError + 143,
	glyph_tangent_vectors_both_zero = gxFirstSystemDebuggingError + 144,
/* validation layout errors */
	layout_run_length_negative	= gxFirstSystemDebuggingError + 145,
	layout_run_length_zero		= gxFirstSystemDebuggingError + 146,
	layout_run_level_negative	= gxFirstSystemDebuggingError + 147,
	layout_run_lengths_do_not_sum_to_text_length = gxFirstSystemDebuggingError + 148,
/* validation picture errors */
	bad_shape_in_picture		= gxFirstSystemDebuggingError + 149,
	bad_style_in_picture		= gxFirstSystemDebuggingError + 150,
	bad_ink_in_picture			= gxFirstSystemDebuggingError + 151,
	bad_transform_in_picture	= gxFirstSystemDebuggingError + 152,
	bad_shape_cache_in_picture	= gxFirstSystemDebuggingError + 153,
	bad_seed_in_picture			= gxFirstSystemDebuggingError + 154,
	invalid_picture_count		= gxFirstSystemDebuggingError + 155,
/* validation text face errors */
	bad_textLayer_count			= gxFirstSystemDebuggingError + 156,
	bad_fillType_in_textFace	= gxFirstSystemDebuggingError + 157,
	bad_style_in_textFace		= gxFirstSystemDebuggingError + 158,
	bad_transform_in_textFace	= gxFirstSystemDebuggingError + 159,
/* validation transform errors */
	invalid_matrix_flag			= gxFirstSystemDebuggingError + 160,
	transform_clip_missing		= gxFirstSystemDebuggingError + 161,
/* validation font cache errors */
	metrics_wrong_type			= gxFirstSystemDebuggingError + 162,
	metrics_point_size_probably_bad = gxFirstSystemDebuggingError + 163,
	scalar_block_wrong_type		= gxFirstSystemDebuggingError + 164,
	scalar_block_parent_mismatch = gxFirstSystemDebuggingError + 165,
	scalar_block_too_small		= gxFirstSystemDebuggingError + 166,
	scalar_block_too_large		= gxFirstSystemDebuggingError + 167,
	invalid_metrics_range		= gxFirstSystemDebuggingError + 168,
	invalid_metrics_flags		= gxFirstSystemDebuggingError + 169,
	metrics_maxWidth_probably_bad = gxFirstSystemDebuggingError + 170,
	font_wrong_type				= gxFirstSystemDebuggingError + 171,
	font_wrong_size				= gxFirstSystemDebuggingError + 172,
	invalid_font_platform		= gxFirstSystemDebuggingError + 173,
	invalid_lookup_range		= gxFirstSystemDebuggingError + 174,
	invalid_lookup_platform		= gxFirstSystemDebuggingError + 175,
	font_not_in_font_list		= gxFirstSystemDebuggingError + 176,
	metrics_not_in_metrics_list	= gxFirstSystemDebuggingError + 177,
/* validation view device errors */
	bad_device_private_flags	= gxFirstSystemDebuggingError + 178,
	bad_device_attributes		= gxFirstSystemDebuggingError + 179,
	invalid_device_number		= gxFirstSystemDebuggingError + 180,
	invalid_device_viewGroup	= gxFirstSystemDebuggingError + 181,
	invalid_device_bounds		= gxFirstSystemDebuggingError + 182,
	invalid_bitmap_in_device	= gxFirstSystemDebuggingError + 183,
/* validation color set errors */
	colorSet_wrong_type			= gxFirstSystemDebuggingError + 184,
	invalid_colorSet_viewDevice_owners = gxFirstSystemDebuggingError + 185,
	invalid_colorSet_colorSpace	= gxFirstSystemDebuggingError + 186,
	invalid_colorSet_count		= gxFirstSystemDebuggingError + 187,
/* validation color profile errors */
	colorProfile_wrong_type		= gxFirstSystemDebuggingError + 188,
	invalid_colorProfile_flags	= gxFirstSystemDebuggingError + 189,
	invalid_colorProfile_response_count = gxFirstSystemDebuggingError + 190,
/* validation internal backing store errors */
	backing_free_parent_mismatch = gxFirstSystemDebuggingError + 191,
	backing_store_parent_mismatch = gxFirstSystemDebuggingError + 192
};

enum gxGraphicWarnings {
/* warnings about warnings */
	warning_stack_underflow		= gxFirstSystemWarning,
	warning_stack_overflow		= gxFirstSystemWarning + 1,
	notice_stack_underflow		= gxFirstSystemWarning + 2,
	notice_stack_overflow		= gxFirstSystemWarning + 3,
	about_to_grow_heap			= gxFirstSystemWarning + 4,
	about_to_unload_objects		= gxFirstSystemWarning + 5,
/* result went out of range */
	map_shape_out_of_range		= gxFirstResultOutOfRangeWarning,
	move_shape_out_of_range		= gxFirstResultOutOfRangeWarning + 1,
	scale_shape_out_of_range	= gxFirstResultOutOfRangeWarning + 2,
	rotate_shape_out_of_range	= gxFirstResultOutOfRangeWarning + 3,
	skew_shape_out_of_range		= gxFirstResultOutOfRangeWarning + 4,
	map_transform_out_of_range	= gxFirstResultOutOfRangeWarning + 5,
	move_transform_out_of_range	= gxFirstResultOutOfRangeWarning + 6,
	scale_transform_out_of_range = gxFirstResultOutOfRangeWarning + 7,
	rotate_transform_out_of_range = gxFirstResultOutOfRangeWarning + 8,
	skew_transform_out_of_range	= gxFirstResultOutOfRangeWarning + 9,
	map_points_out_of_range		= gxFirstResultOutOfRangeWarning + 10,
/* gave a parameter out of range */
	contour_out_of_range		= gxFirstParameterOutOfRangeWarning,
	index_out_of_range_in_contour = gxFirstParameterOutOfRangeWarning + 1,
	picture_index_out_of_range	= gxFirstParameterOutOfRangeWarning + 2,
	color_index_requested_not_found = gxFirstParameterOutOfRangeWarning + 3,
	colorSet_index_out_of_range	= gxFirstParameterOutOfRangeWarning + 4,
	index_out_of_range			= gxFirstParameterOutOfRangeWarning + 5,
	count_out_of_range			= gxFirstParameterOutOfRangeWarning + 6,
	length_out_of_range			= gxFirstParameterOutOfRangeWarning + 7,
	font_table_index_out_of_range = gxFirstParameterOutOfRangeWarning + 8,
	font_glyph_index_out_of_range = gxFirstParameterOutOfRangeWarning + 9,
	point_out_of_range			= gxFirstParameterOutOfRangeWarning + 10,
	profile_response_out_of_range = gxFirstParameterOutOfRangeWarning + 11,
/* gxFont scaler warnings */
	font_scaler_no_output		= gxFirstFontScalerWarning,
	font_scaler_fake_metrics	= gxFirstFontScalerWarning + 1,
	font_scaler_fake_linespacing = gxFirstFontScalerWarning + 2,
	font_scaler_glyph_substitution = gxFirstFontScalerWarning + 3,
	font_scaler_no_kerning_applied = gxFirstFontScalerWarning + 4,
/* might not be what you expected */
	character_substitution_took_place = gxFirstFontScalerWarning + 5,
	unable_to_get_bounds_on_multiple_devices = gxFirstFontScalerWarning + 6,
	font_language_not_found		= gxFirstFontScalerWarning + 7,
	font_not_found_during_unflattening = gxFirstFontScalerWarning + 8,
/*storage */
	unrecognized_stream_version	= gxFirstFontScalerWarning + 9,
	bad_data_in_stream			= gxFirstFontScalerWarning + 10
};

enum {
/* nonsense data */
	new_shape_contains_invalid_data = gxFirstSystemDebuggingWarning,
	new_tag_contains_invalid_data = gxFirstSystemDebuggingWarning + 1,
	extra_data_passed_was_ignored = gxFirstSystemDebuggingWarning + 2,
	font_table_not_found		= gxFirstSystemDebuggingWarning + 3,
	font_name_not_found			= gxFirstSystemDebuggingWarning + 4,
/* doesn't make sense to do */
	unable_to_traverse_open_contour_that_starts_or_ends_off_the_curve = gxFirstSystemDebuggingWarning + 5,
	unable_to_draw_open_contour_that_starts_or_ends_off_the_curve = gxFirstSystemDebuggingWarning + 6,
	cannot_dispose_default_shape = gxFirstSystemDebuggingWarning + 7,
	cannot_dispose_default_style = gxFirstSystemDebuggingWarning + 8,
	cannot_dispose_default_ink	= gxFirstSystemDebuggingWarning + 9,
	cannot_dispose_default_transform = gxFirstSystemDebuggingWarning + 10,
	cannot_dispose_default_colorProfile = gxFirstSystemDebuggingWarning + 11,
	cannot_dispose_default_colorSet = gxFirstSystemDebuggingWarning + 12,
	shape_direct_attribute_not_set = gxFirstSystemDebuggingWarning + 13,
/* couldn't find what you were looking for */
	point_does_not_intersect_port = gxFirstSystemDebuggingWarning + 14,
	cannot_dispose_non_font		= gxFirstSystemDebuggingWarning + 15,
	face_override_style_font_must_match_style = gxFirstSystemDebuggingWarning + 16,
	union_of_area_and_length_returns_area_only = gxFirstSystemDebuggingWarning + 17,
	insufficient_coordinate_space_for_new_device = gxFirstSystemDebuggingWarning + 18,
/* other */
	shape_passed_has_no_bounds	= gxFirstSystemDebuggingWarning + 19,
	tags_of_type_flst_removed	= gxFirstSystemDebuggingWarning + 20,
	translator_not_installed_on_this_grafport = gxFirstSystemDebuggingWarning + 21
};

enum gxGraphicNotices {
	parameters_have_no_effect	= gxFirstSystemNotice,
	attributes_already_set		= gxFirstSystemNotice + 1,
	caps_already_set			= gxFirstSystemNotice + 2,
	clip_already_set			= gxFirstSystemNotice + 3,
	color_already_set			= gxFirstSystemNotice + 4,
	curve_error_already_set		= gxFirstSystemNotice + 5,
	dash_already_set			= gxFirstSystemNotice + 6,
	default_colorProfile_already_set = gxFirstSystemNotice + 7,
	default_ink_already_set		= gxFirstSystemNotice + 8,
	default_transform_already_set = gxFirstSystemNotice + 9,
	default_shape_already_set	= gxFirstSystemNotice + 10,
	default_style_already_set	= gxFirstSystemNotice + 11,
	dither_already_set			= gxFirstSystemNotice + 12,
	encoding_already_set		= gxFirstSystemNotice + 13,
	face_already_set			= gxFirstSystemNotice + 14,
	fill_already_set			= gxFirstSystemNotice + 15,
	font_already_set			= gxFirstSystemNotice + 16,
	font_variations_already_set	= gxFirstSystemNotice + 17,
	glyph_positions_are_already_set = gxFirstSystemNotice + 18,
	glyph_tangents_are_already_set = gxFirstSystemNotice + 19,
	halftone_already_set		= gxFirstSystemNotice + 20,
	hit_test_already_set		= gxFirstSystemNotice + 21,
	ink_already_set				= gxFirstSystemNotice + 22,
	join_already_set			= gxFirstSystemNotice + 23,
	justification_already_set	= gxFirstSystemNotice + 24,
	mapping_already_set			= gxFirstSystemNotice + 25,
	pattern_already_set			= gxFirstSystemNotice + 26,
	pen_already_set				= gxFirstSystemNotice + 27,
	style_already_set			= gxFirstSystemNotice + 28,
	tag_already_set				= gxFirstSystemNotice + 29,
	text_attributes_already_set	= gxFirstSystemNotice + 30,
	text_size_already_set		= gxFirstSystemNotice + 31,
	transfer_already_set		= gxFirstSystemNotice + 32,
	translator_already_installed_on_this_grafport = gxFirstSystemNotice + 33,
	transform_already_set		= gxFirstSystemNotice + 34,
	type_already_set			= gxFirstSystemNotice + 35,
	validation_level_already_set = gxFirstSystemNotice + 36,
	viewPorts_already_set		= gxFirstSystemNotice + 37,
	viewPort_already_in_viewGroup = gxFirstSystemNotice + 38,
	viewDevice_already_in_viewGroup = gxFirstSystemNotice + 39,
	geometry_unaffected			= gxFirstSystemNotice + 40,
	mapping_unaffected			= gxFirstSystemNotice + 41,
	tags_in_shape_ignored		= gxFirstSystemNotice + 42,
	shape_already_in_primitive_form = gxFirstSystemNotice + 43,
	shape_already_in_simple_form = gxFirstSystemNotice + 44,
	shape_already_broken		= gxFirstSystemNotice + 45,
	shape_already_joined		= gxFirstSystemNotice + 46,
	cache_already_cleared		= gxFirstSystemNotice + 47,
	shape_not_disposed			= gxFirstSystemNotice + 48,
	style_not_disposed			= gxFirstSystemNotice + 49,
	ink_not_disposed			= gxFirstSystemNotice + 50,
	transform_not_disposed		= gxFirstSystemNotice + 51,
	colorSet_not_disposed		= gxFirstSystemNotice + 52,
	colorProfile_not_disposed	= gxFirstSystemNotice + 53,
	font_not_disposed			= gxFirstSystemNotice + 54,
	glyph_tangents_have_no_effect = gxFirstSystemNotice + 55,
	glyph_positions_determined_by_advance = gxFirstSystemNotice + 56,
	transform_viewPorts_already_set = gxFirstSystemNotice + 57,
	directShape_attribute_set_as_side_effect = gxFirstSystemNotice + 58,
	lockShape_called_as_side_effect = gxFirstSystemNotice + 59,
	lockTag_called_as_side_effect = gxFirstSystemNotice + 60,
	shapes_unlocked_as_side_effect = gxFirstSystemNotice + 61,
	shape_not_locked			= gxFirstSystemNotice + 62,
	tag_not_locked				= gxFirstSystemNotice + 63,
	profile_not_locked			= tag_not_locked,
	lockProfile_called_as_side_effect = lockTag_called_as_side_effect,
	disposed_dead_caches		= gxFirstSystemNotice + 64,
	disposed_live_caches		= gxFirstSystemNotice + 65,
	low_on_memory				= gxFirstSystemNotice + 66,
	very_low_on_memory			= gxFirstSystemNotice + 67,
	transform_references_disposed_viewPort = gxFirstSystemNotice + 68
};

typedef long gxGraphicsError;

typedef long gxGraphicsWarning;

typedef long gxGraphicsNotice;

typedef void (*gxUserErrorProcPtr)(gxGraphicsError status, long refcon);
typedef void (*gxUserWarningProcPtr)(gxGraphicsWarning status, long refcon);
typedef void (*gxUserNoticeProcPtr)(gxGraphicsNotice status, long refcon);

#if GENERATINGCFM
typedef UniversalProcPtr gxUserErrorUPP;
typedef UniversalProcPtr gxUserWarningUPP;
typedef UniversalProcPtr gxUserNoticeUPP;
#else
typedef gxUserErrorProcPtr gxUserErrorUPP;
typedef gxUserWarningProcPtr gxUserWarningUPP;
typedef gxUserNoticeProcPtr gxUserNoticeUPP;
#endif

enum {
	uppgxUserErrorProcInfo = kCStackBased
		 | STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(gxGraphicsError)))
		 | STACK_ROUTINE_PARAMETER(2, SIZE_CODE(sizeof(long))),
	uppgxUserWarningProcInfo = kCStackBased
		 | STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(gxGraphicsWarning)))
		 | STACK_ROUTINE_PARAMETER(2, SIZE_CODE(sizeof(long))),
	uppgxUserNoticeProcInfo = kCStackBased
		 | STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(gxGraphicsNotice)))
		 | STACK_ROUTINE_PARAMETER(2, SIZE_CODE(sizeof(long)))
};

#if GENERATINGCFM
#define NewgxUserErrorProc(userRoutine)		\
		(gxUserErrorUPP) NewRoutineDescriptor((ProcPtr)(userRoutine), uppgxUserErrorProcInfo, GetCurrentArchitecture())
#define NewgxUserWarningProc(userRoutine)		\
		(gxUserWarningUPP) NewRoutineDescriptor((ProcPtr)(userRoutine), uppgxUserWarningProcInfo, GetCurrentArchitecture())
#define NewgxUserNoticeProc(userRoutine)		\
		(gxUserNoticeUPP) NewRoutineDescriptor((ProcPtr)(userRoutine), uppgxUserNoticeProcInfo, GetCurrentArchitecture())
#else
#define NewgxUserErrorProc(userRoutine)		\
		((gxUserErrorUPP) (userRoutine))
#define NewgxUserWarningProc(userRoutine)		\
		((gxUserWarningUPP) (userRoutine))
#define NewgxUserNoticeProc(userRoutine)		\
		((gxUserNoticeUPP) (userRoutine))
#endif

#if GENERATINGCFM
#define CallgxUserErrorProc(userRoutine, status, refcon)		\
		CallUniversalProc((UniversalProcPtr)(userRoutine), uppgxUserErrorProcInfo, (status), (refcon))
#define CallgxUserWarningProc(userRoutine, status, refcon)		\
		CallUniversalProc((UniversalProcPtr)(userRoutine), uppgxUserWarningProcInfo, (status), (refcon))
#define CallgxUserNoticeProc(userRoutine, status, refcon)		\
		CallUniversalProc((UniversalProcPtr)(userRoutine), uppgxUserNoticeProcInfo, (status), (refcon))
#else
#define CallgxUserErrorProc(userRoutine, status, refcon)		\
		(*(userRoutine))((status), (refcon))
#define CallgxUserWarningProc(userRoutine, status, refcon)		\
		(*(userRoutine))((status), (refcon))
#define CallgxUserNoticeProc(userRoutine, status, refcon)		\
		(*(userRoutine))((status), (refcon))
#endif

typedef gxUserErrorProcPtr gxUserErrorFunction;

typedef gxUserWarningProcPtr gxUserWarningFunction;

typedef gxUserNoticeProcPtr gxUserNoticeFunction;


enum {
	common_colors_not_initialized = gxFirstLibraryError,
	no_open_picture				= gxFirstLibraryError + 1,
	picture_already_open		= gxFirstLibraryError + 2,
	no_open_poly				= gxFirstLibraryError + 3,
	poly_already_open			= gxFirstLibraryError + 4,
	no_open_region				= gxFirstLibraryError + 5,
	region_already_open			= gxFirstLibraryError + 6,
	no_active_picture			= gxFirstLibraryError + 7
};

enum {
	no_picture_drawn			= gxFirstLibraryWarning,
	polygons_have_different_size_contours = gxFirstLibraryWarning + 1,
	graphic_type_cannot_be_specifed_by_four_values = gxFirstLibraryWarning + 2,
	graphic_type_cannot_be_specifed_by_six_values = gxFirstLibraryWarning + 3,
	point_expected				= gxFirstLibraryWarning + 4,
	line_or_rectangle_expected	= gxFirstLibraryWarning + 5,
	curve_expected				= gxFirstLibraryWarning + 6,
	graphic_type_does_not_contain_control_bits = gxFirstLibraryWarning + 7,
	request_exceeds_available_data = gxFirstLibraryWarning + 8,
	extra_data_unread			= gxFirstLibraryWarning + 9,
	no_variable_length_user_data_saved = gxFirstLibraryWarning + 10
};

enum {
	zero_length_string_passed	= gxFirstLibraryNotice
};

#define graphicsDebuggingIncludes
/* old header = graphics debugging */
 
enum gxValidationLevels {
/* These levels tell how to validate routines.  Choose one. */
	gxNoValidation				= 0x00,							/* no validation */
	gxPublicValidation			= 0x01,							/* check parameters to public routines */
	gxInternalValidation		= 0x02,							/* check parameters to internal routines */
/* These levels tell how to validate types.  Choose one. */
	gxTypeValidation			= 0x00,							/* check types of objects */
	gxStructureValidation		= 0x10,							/* check fields of private structures */
	gxAllObjectValidation		= 0x20,							/* check every object over every call */
/* These levels tell how to validate memory manager blocks.  Choose any combination. */
	gxNoMemoryManagerValidation	= 0x0000,
	gxApBlockValidation			= 0x0100,						/* check the relevant block structures after each memory mgr. call */
	gxFontBlockValidation		= 0x0200,						/* check the system gxHeap as well */
	gxApHeapValidation			= 0x0400,						/* check the memory managers gxHeap after every mem. call */
	gxFontHeapValidation		= 0x0800,						/* check the system gxHeap as well */
	gxCheckApHeapValidation		= 0x1000,						/* check the memory managers gxHeap if checking routine parameters */
	gxCheckFontHeapValidation	= 0x2000						/* check the system gxHeap as well */
};

typedef long gxValidationLevel;


enum gxDrawErrors {
	no_draw_error				= 0,
/* gxShape type errors */
	shape_emptyType				= 1,
	shape_inverse_fullType		= 2,
	rectangle_zero_width		= 3,
	rectangle_zero_height		= 4,
	polygon_empty				= 5,
	path_empty					= 6,
	bitmap_zero_width			= 7,
	bitmap_zero_height			= 8,
	text_empty					= 9,
	glyph_empty					= 10,
	layout_empty				= 11,
	picture_empty				= 12,
/* general gxShape errors */
	shape_no_fill				= 13,
	shape_no_enclosed_area		= 14,
	shape_no_enclosed_pixels	= 15,
	shape_very_small			= 16,
	shape_very_large			= 17,
	shape_contours_cancel		= 18,
/* gxStyle errors */
	pen_too_small				= 19,
	text_size_too_small			= 20,
	dash_empty					= 21,
	start_cap_empty				= 22,
	pattern_empty				= 23,
	textFace_empty				= 24,
	shape_primitive_empty		= 25,
	shape_primitive_very_small	= 26,
/* gxInk errors */
	transfer_equals_noMode		= 27,
	transfer_matrix_ignores_source = 28,
	transfer_matrix_ignores_device = 29,
	transfer_source_reject		= 30,
	transfer_mode_ineffective	= 31,
	colorSet_no_entries			= 32,
	bitmap_colorSet_one_entry	= 33,
/* gxTransform errors */
	transform_scale_too_small	= 34,
	transform_map_too_large		= 35,
	transform_move_too_large	= 36,
	transform_scale_too_large	= 37,
	transform_rotate_too_large	= 38,
	transform_perspective_too_large = 39,
	transform_skew_too_large	= 40,
	transform_clip_no_intersection = 41,
	transform_clip_empty		= 42,
	transform_no_viewPorts		= 43,
/* gxViewPort errors */
	viewPort_disposed			= 44,
	viewPort_clip_empty			= 45,
	viewPort_clip_no_intersection = 46,
	viewPort_scale_too_small	= 47,
	viewPort_map_too_large		= 48,
	viewPort_move_too_large		= 49,
	viewPort_scale_too_large	= 50,
	viewPort_rotate_too_large	= 51,
	viewPort_perspective_too_large = 52,
	viewPort_skew_too_large		= 53,
	viewPort_viewGroup_offscreen = 54,
/* gxViewDevice errors */
	viewDevice_clip_no_intersection = 55,
	viewDevice_scale_too_small	= 56,
	viewDevice_map_too_large	= 57,
	viewDevice_move_too_large	= 58,
	viewDevice_scale_too_large	= 59,
	viewDevice_rotate_too_large	= 60,
	viewDevice_perspective_too_large = 61,
	viewDevice_skew_too_large	= 62
};

typedef long gxDrawError;

typedef Boolean (*gxUserDebugFunction)(const char *str, long message, long reference);
extern gxDrawError GXGetShapeDrawError(gxShape source)
 THREEWORDINLINE(0x303C, 0x1ee, 0xA832);
extern void GXValidateAll(void)
 THREEWORDINLINE(0x303C, 0x1ef, 0xA832);
extern void GXValidateColorSet(gxColorSet target)
 THREEWORDINLINE(0x303C, 0x1f0, 0xA832);
extern void GXValidateColorProfile(gxColorProfile target)
 THREEWORDINLINE(0x303C, 0x1f1, 0xA832);
extern void GXValidateGraphicsClient(gxGraphicsClient target)
 THREEWORDINLINE(0x303C, 0x1f2, 0xA832);
extern void GXValidateInk(gxInk target)
 THREEWORDINLINE(0x303C, 0x1f3, 0xA832);
extern void GXValidateShape(gxShape target)
 THREEWORDINLINE(0x303C, 0x1f4, 0xA832);
extern void GXValidateStyle(gxStyle target)
 THREEWORDINLINE(0x303C, 0x1f5, 0xA832);
extern void GXValidateTag(gxTag target)
 THREEWORDINLINE(0x303C, 0x1f6, 0xA832);
extern void GXValidateTransform(gxTransform target)
 THREEWORDINLINE(0x303C, 0x1f7, 0xA832);
extern void GXValidateViewDevice(gxViewDevice target)
 THREEWORDINLINE(0x303C, 0x1f8, 0xA832);
extern void GXValidateViewPort(gxViewPort target)
 THREEWORDINLINE(0x303C, 0x1f9, 0xA832);
extern void GXValidateViewGroup(gxViewGroup target)
 THREEWORDINLINE(0x303C, 0x1fa, 0xA832);
extern gxValidationLevel GXGetValidation(void)
 THREEWORDINLINE(0x303C, 0x1fb, 0xA832);
extern void GXSetValidation(gxValidationLevel level)
 THREEWORDINLINE(0x303C, 0x1fc, 0xA832);
extern long GXGetValidationError(char *procedureName, void **argument, long *argumentNumber)
 THREEWORDINLINE(0x303C, 0x1fd, 0xA832);
extern gxUserDebugFunction GXGetUserGraphicsDebug(long *reference)
 THREEWORDINLINE(0x303C, 0x1ff, 0xA832);
extern void GXSetUserGraphicsDebug(gxUserDebugFunction userFunction, long reference)
 THREEWORDINLINE(0x303C, 0x200, 0xA832);
 
#if defined(__MWERKS__) && GENERATING68K
	#pragma pop
#endif

#if PRAGMA_IMPORT_SUPPORTED
#pragma import off
#endif

#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=reset
#endif

#ifdef __cplusplus
}
#endif

#endif /* __GXERRORS__ */
